@@ -9,16 +9,17 @@
 #define GPMISCDIR "/usr/local/share/pari"
 #define SHELL_Q '\''
 
-#define PARIVERSION "GP/PARI CALCULATOR Version 2.1.6 (released)"
+#define PARIVERSION "GP/PARI CALCULATOR Version 2.1.7 (released)"
 #ifdef __cplusplus
-# define PARIINFO "C++ Alpha 64-bit version"
+# define PARIINFO "C++ armv7 (C portable kernel) 32-bit version"
 #else
-# define PARIINFO "Alpha 64-bit version"
+# define PARIINFO "armv7 (C portable kernel) 32-bit version"
 #endif
-#define PARI_VERSION_CODE 131334
+#define PARI_VERSION_CODE 131335
 #define PARI_VERSION(a,b,c) (((a) << 16) + ((b) << 8) + (c))
 
-#define PARI_DOUBLE_FORMAT -
+#define PARI_DOUBLE_FORMAT 1
+#define __HAS_NO_ASM__
 #define ASMINLINE
 
 /*  Location of GNU gzip program (enables reading of .Z and .gz files). */
@@ -36,12 +37,12 @@
 #define HAS_RL_COMPLETION_MATCHES
 #define HAS_RL_COMPLETION_FUNC_T
 
-#define LONG_IS_64BIT
 #define NOEXP2
 #define USE_GETRUSAGE 1
 #define USE_SIGSETMASK 1
 #define HAS_DLOPEN
 #define DL_DFLT_NAME "libpari."
+#define STACK_CHECK
 #define HAS_TIOCGWINSZ
 #define HAS_STRFTIME
 #define HAS_OPENDIR
