@@ -1,4 +1,4 @@
-/* $Id: base4.c,v 1.42 2000/11/03 21:00:20 karim Exp $
+/* $Id: base4.c,v 1.42.2.2 2002/12/04 21:01:47 bill Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -1312,27 +1312,66 @@ idealinv(GEN nf, GEN x)
   res[1]=(long)x; res[2]=lneg(ax); return res;
 }
 
+/* return x such that vp^n = x/d */
+static GEN
+idealpowprime_spec(GEN nf, GEN vp, GEN n, GEN *d)
+{
+  GEN n1, x, r;
+  long s = signe(n);
+
+  if (s == 0) err(talker, "0th power in idealpowprime_spec");
+  if (s < 0) n = negi(n);
+  /* now n > 0 */
+  x = dummycopy(vp);
+  n1 = dvmdii(n, (GEN)x[3], &r);
+  if (signe(r)) n1 = addis(n1,1); /* n1 = ceil(n/e) */
+  x[1] = (long)powgi((GEN)x[1],n1);
+  if (s < 0)
+  {
+    x[2] = ldiv(element_pow(nf,(GEN)x[5],n), powgi((GEN)vp[1],subii(n,n1)));
+    *d = (GEN)x[1];
+  }
+  else
+  {
+    x[2] = (long)element_pow(nf,(GEN)x[2],n);
+    *d = NULL;
+  }
+  return x;
+}
+
 static GEN
 idealpowprime(GEN nf, GEN vp, GEN n)
 {
-  GEN n1, x;
+  GEN x, d;
   long s = signe(n);
 
   nf = checknf(nf);
   if (s == 0) return idmat(lgef(nf[1])-3);
-  if (s < 0) n = negi(n);
-  x = dummycopy(vp);
-  n1 = gceil(gdiv(n,(GEN)x[3]));
-  x[1]=(long)powgi((GEN)x[1],n1);
-  if (s < 0)
-    x[2]=ldiv(element_pow(nf,(GEN)x[5],n), powgi((GEN)vp[1],subii(n,n1)));
-  else
-    x[2]=(long)element_pow(nf,(GEN)x[2],n);
+  x = idealpowprime_spec(nf, vp, n, &d);
   x = prime_to_ideal_aux(nf,x);
-  if (s<0) x = gdiv(x, powgi((GEN)vp[1],n1));
+  if (d) x = gdiv(x, d);
   return x;
 }
 
+/* x * vp^n */
+GEN
+idealmulpowprime(GEN nf, GEN x, GEN vp, GEN n)
+{
+  GEN denx,y,d;
+
+  if (!signe(n)) return x;
+  nf = checknf(nf);
+  y = idealpowprime_spec(nf, vp, n, &d);
+  denx = denom(x);
+  if (gcmp1(denx)) denx = d; else
+  {
+    x = gmul(denx,x);
+    if (d) denx = mulii(d,denx);
+  }
+  x = idealmulspec(nf,x, (GEN)y[1], (GEN)y[2]);
+  return denx? gdiv(x,denx): x;
+}
+
 /* raise the ideal x to the power n (in Z) */
 GEN
 idealpow(GEN nf, GEN x, GEN n)
@@ -1748,6 +1787,13 @@ appr_reduce(GEN s, GEN y, long N)
   return gadd(s, gmul(y,u));
 }
 
+static void
+check_listpr(GEN x)
+{
+  long l = lg(x), i;
+  for (i=1; i<l; i++) checkprimeid((GEN)x[i]);
+}
+
 /* Given a fractional ideal x (if fl=0) or a prime ideal factorization
  * with possibly zero or negative exponents (if fl=1), gives a b such that
  * v_p(b)=v_p(x) for all prime ideals p dividing x (or in the factorization)
@@ -1770,6 +1816,7 @@ idealappr0(GEN nf, GEN x, long fl)
   {
     if (typ(x)!=t_MAT || lg(x)!=3)
       err(talker,"not a prime ideal factorization in idealappr0");
+    check_listpr((GEN)x[1]);
     fact=x; list=(GEN)fact[1]; ep=(GEN)fact[2]; r=lg(list);
     if (r==1) return gscalcol_i(gun,N);
     for (i=1; i<r; i++)
