@@ -1,4 +1,4 @@
-/* $Id: bibli2.c,v 1.17 2000/11/03 21:00:21 karim Exp $
+/* $Id: bibli2.c,v 1.17.2.2 2001/06/14 19:34:20 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -986,19 +986,22 @@ GEN
 permute(long n, GEN x)
 {
   long av=avma,i,a,r;
-  GEN v,w,y;
+  GEN v,w;
 
   if (n < 1) err(talker,"n too small (%ld) in numtoperm",n);
-  v=(GEN)gpmalloc((n+1)*sizeof(long)); v[1]=1;
-  for (r=2; r<=n; r++)
+  if (typ(x) != t_INT) err(arither1); 
+  v = cgetg(n+1, t_VEC);
+  v[1]=1; av = avma;
+  if (signe(x) <= 0) x = modii(x, mpfact(n));
+  for (r=2; r<=n; r++) 
   {
-    x=dvmdis(x,r,&w); a=itos(w);
-    for (i=r; i>=a+2; i--) v[i]=v[i-1];
+    x = dvmdis(x,r,&w); a = itos(w);
+    for (i=r; i>=a+2; i--) v[i] = v[i-1];
     v[i]=r;
   }
-  avma=av; y=cgetg(n+1,t_VEC);
-  for (i=1; i<=n; i++) y[i]=lstoi(v[i]);
-  free(v); return y;
+  avma = av;
+  for (i=1; i<=n; i++) v[i] = lstoi(v[i]);
+  return v;
 }
 
 GEN
