@@ -1,4 +1,4 @@
-/* $Id: buch2.c,v 1.65 2000/11/08 19:12:21 karim Exp $
+/* $Id: buch2.c,v 1.65.2.5 2004/05/05 08:46:44 kb Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -670,6 +670,8 @@ get_norm_fact(GEN gen, GEN ex, GEN *pd)
   *pd = d; return N;
 }
 
+extern GEN idealmulpowprime(GEN nf, GEN x, GEN vp, GEN n);
+
 /* try to split ideal / (some integer) on FB */
 static long
 factorgensimple(GEN nf,GEN ideal)
@@ -721,6 +723,18 @@ factorgensimple(GEN nf,GEN ideal)
         lo++; primfact[lo] = l; P = (GEN)vectbase[l];
         expoprimfact[lo] = - k * itos((GEN)P[3]);
       }
+      /* check that ideal / p^k / (tentative factorisation) is integral */
+      {
+        GEN z = ideal;
+        for (l = lo0+1; l <= lo; l++)
+        {
+          GEN n = stoi(- expoprimfact[l]);
+          z = idealmulpowprime(nf, z, (GEN)vectbase[primfact[l]], n);
+        }
+        z = gdiv(z, gpowgs(p, k));
+        if (!gcmp1(denom(z))) { avma=av1; return 0; }
+        ideal = z;
+      }
     }
     if (gcmp1(y)) { avma=av1; primfact[0]=lo; return 1; }
     x = y;
@@ -954,10 +968,11 @@ static long
 prec_arch(GEN bnf)
 {
   GEN a = (GEN)bnf[4];
-  long i,prec;
+  long i, l = lg(a), prec;
 
-  for (i=1; ; i++)
+  for (i=1; i<l; i++)
     if ( (prec = gprecision((GEN)a[i])) ) return prec;
+  return DEFAULTPREC;
 }
 
 /* col = archimedian components of x, Nx its norm, dx a bound for its
@@ -1785,10 +1800,11 @@ be_honest(GEN nf,GEN subFB,long RU,long PRECREGINT)
     P = idealbase[numFB[FB[iz]]];
     J = lg(P);
     /* if unramified, check all but 1 */
-    if (!divise(D, gmael(P,1,1))) J--;
+    if (J > 1 && !divise(D, gmael(P,1,1))) J--;
     for (j=1; j<J; j++)
     {
       GEN ideal0 = prime_to_ideal(nf,(GEN)P[j]);
+      ulong av2 = avma;
       for(nbtest=0;;)
       {
 	ideal = ideal0;
@@ -1814,7 +1830,7 @@ be_honest(GEN nf,GEN subFB,long RU,long PRECREGINT)
 	      factorgen(nf,idealpro,iz-1,FB[iz-1])) break;
 	  nbtest++; if (nbtest==200) return 0;
 	}
-	avma=av; if (k <= RU) break;
+	avma=av2; if (k <= RU) break;
       }
     }
   }
@@ -2906,7 +2922,7 @@ INCREASEGEN:
 
   /* random relations */
   if (cglob == KCCO) /* enough relations, initialize nevertheless */
-    ((void(*)(long))random_relation)(-1);
+    random_relation(-1,0,0,0,0,0,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
   else
   {
     GEN maarch;
@@ -3027,11 +3043,13 @@ INCREASEGEN:
     }
     W = hnfadd(W,vperm,&pdep,&B,&C, extramat,extraC);
     nlze = lg(pdep)>1? lg(pdep[1])-1: lg(B[1])-1;
+#if 0 /* useless now: rank nearly maximal. Random relations are preferable */
     if (nlze)
     {
       list_jideal = cgetg(nlze+1, t_VECSMALL);
       for (i=1; i<=nlze; i++) list_jideal[i] = vperm[i];
     }
+#endif
     KCCOPRO += extrarel;
     if (nlze && ++nreldep > MAXRELSUP) { sfb_increase=1; goto LABELINT; }
   }
