@@ -1,4 +1,4 @@
-/* $Id: buch3.c,v 1.15 2000/11/03 21:00:21 karim Exp $
+/* $Id: buch3.c,v 1.15.2.3 2002/03/29 18:51:10 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -54,7 +54,7 @@ get_full_rank(GEN nf, GEN v, GEN _0, GEN _1, GEN vecsign, GEN gen,
       for (i=1; i<=rankmax; i++)
 	vecsign[i] = (gsigne(gsubst(alpha,va,(GEN)rac[i])) > 0)? (long)_0
                                                                : (long)_1;
-      v1 = concatsp(v, vecsign);
+      v1 = concat(v, vecsign);
       if (rank(v1) == rankinit) avma=av1;
       else
       {
@@ -612,7 +612,7 @@ zimmertbound(long N,long R2,GEN DK)
 {/*20*/28.1285704, 27.4021674, 26.6807314, 25.9645140, 25.2537867, 24.5488420,
        23.8499943, 23.1575823, 22.4719720, 21.7935548, 21.1227537}
     };
-    w = gmul(dbltor(exp(-c[N][R2])), gsqrt(DK,MEDDEFAULTPREC));
+    w = gmul(dbltor(exp(-c[N-2][R2])), gsqrt(DK,MEDDEFAULTPREC));
   }
   else
   {
@@ -791,25 +791,25 @@ is_unit(GEN M, long r1, GEN x)
 #define NBMAX 5000
 /* should use smallvectors */
 static GEN
-minimforunits(GEN nf, long BOUND, long stockmax)
+minimforunits(GEN nf, long BORNE, long stockmax)
 {
   const long prec = MEDDEFAULTPREC;
   long av = avma,n,i,j,k,s,norme,normax,*x,cmpt,r1;
   GEN u,r,S,a,M,p1;
   double p;
   double **q,*v,*y,*z;
-  double eps=0.000001;
+  double eps=0.000001, BOUND = BORNE * 1.00001;
 
   if (DEBUGLEVEL>=2)
   {
     fprintferr("Searching minimum of T2-form on units:\n");
-    if (DEBUGLEVEL>2) fprintferr("   BOUND = %ld\n",BOUND);
+    if (DEBUGLEVEL>2) fprintferr("   BOUND = %ld\n",BORNE);
     flusherr();
   }
   r1 = itos(gmael(nf,2,1));
   a = gmael(nf,5,3); n = lg(a);
   minim_alloc(n, &q, &x, &y, &z, &v);
-  n--; BOUND += (long)eps;
+  n--;
   u = lllgram(a,prec);
   M = gmul(gmael(nf,5,1), u); /* embeddings of T2-reduced basis */
   M = gprec_w(M, prec);
