@@ -1,4 +1,4 @@
-/* $Id: polarit3.c,v 1.18 2000/11/03 21:00:22 karim Exp $
+/* $Id: polarit3.c,v 1.18.2.2 2002/05/15 16:01:59 bill Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -1086,7 +1086,7 @@ extern GEN polpol_to_mat(GEN v, long n);
 GEN Fp_factor_irred(GEN P,GEN l, GEN Q)
 {
   ulong ltop=avma,av;
-  GEN SP,SQ,MP,MQ,M,MF,E,V,MPT,res;
+  GEN SP,SQ,MP,MQ,M,MF,E,V,IR,res;
   long np=degree(P),nq=degree(Q);
   long i,d=cgcd(np,nq);
   long vp=varn(P),vq=varn(Q);
@@ -1098,16 +1098,17 @@ GEN Fp_factor_irred(GEN P,GEN l, GEN Q)
   }
   MF=matrixpow(nq,nq,Fp_pow_mod_pol(polx[vq],l,Q,l),Q,l);
   Fp_intersect(d,P,Q,l,&SP,&SQ,NULL,MF);
+  av=avma;
   E=Fp_factorgalois(P,l,d,vq);
   E=polpol_to_mat(E,np);
-  av=avma;
   MP = matrixpow(np,d,SP,P,l);
-  MQ = matrixpow(nq,d,SQ,Q,l);
-  MPT=gtrans(MP);
-  M = FpM_mul(MPT,MP,NULL);
-  M = gmul(M,gmodulcp(gun,l));
+  M = gmul(MP,gmodulcp(gun,l));
+  IR = indexrank(M);
+  E = gtrans(extract(gtrans(E),(GEN)IR[1]));
+  M = gtrans(extract(gtrans(M),(GEN)IR[1]));
   M = lift(invmat(M));
-  M = FpM_mul(MQ,FpM_mul(M,MPT,NULL),NULL);
+  MQ = matrixpow(nq,d,SQ,Q,l);
+  M = FpM_mul(MQ,M,l);
   M = FpM_mul(M,E,l);
   M = gerepileupto(av,M);
   V = cgetg(d+1,t_VEC);
@@ -1629,6 +1630,8 @@ modulargcd(GEN a, GEN b)
   B = content(b); D = ggcd(A,B);
   A = gcmp1(A)? a: gdiv(a,A); nA=lgef(A)-3;
   B = gcmp1(B)? b: gdiv(b,B); nB=lgef(B)-3;
+  check_pol_int(A); check_pol_int(B);
+  if (varn(A) != varn(B)) err(talker,"different variables in modulargcd");
   g = mppgcd((GEN)A[nA+2], (GEN)B[nB+2]);
   av2=avma; n=1+min(nA,nB);
   ma=maxnorm(A); mb=maxnorm(B);
