@@ -1,4 +1,4 @@
-/* $Id: rootpol.c,v 1.21 2000/11/03 21:00:23 karim Exp $
+/* $Id: rootpol.c,v 1.21.2.4 2002/01/10 21:50:18 cvsadmin Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -357,24 +357,28 @@ mylog2(GEN z)
   return x+0.5*log2( 1 + exp2(2*(y-x)));
 }
 
+/* find s such that  A_h <= 2^s <= 2 A_i  for one h and all i < n = deg(p),
+ * with  A_i := (binom(n,i) lc(p) / p_i) ^ 1/(n-i), and  p = sum p_i X^i */
 static long
 findpower(GEN p)
 {
-  double x, logbinomial,pente,pentemax=-pariINFINITY;
-  long n=lgef(p)-3,i;
+  double x, logbinomial, mins = pariINFINITY;
+  long n = lgef(p)-3,i;
 
-  logbinomial = mylog2((GEN) p[n+2]);
+  logbinomial = mylog2((GEN)p[n+2]); /* log2(lc * binom(n,i)) */
   for (i=n-1; i>=0; i--)
-  {
+  { 
     logbinomial += log2((double) (i+1) / (double) (n-i));
-    x = mylog2((GEN) p[2+i])-logbinomial;
-    if (x>-pariINFINITY)
+    x = mylog2((GEN)p[i+2]);
+    if (x != -pariINFINITY)
     {
-      pente = x/ (double) (n-i);
-      if (pente > pentemax) pentemax = pente;
+      double s = (logbinomial - x) / (double) (n-i);
+      if (s < mins) mins = s;
     }
   }
-  return (long) -floor(pentemax);
+  i = (long)ceil(mins);
+  if (i - mins > 1 - 1e-12) i--;
+  return i;
 }
 
 /* returns the exponent for the procedure modulus, from the newton diagram */
@@ -615,9 +619,12 @@ lower_bound(GEN p, long *k, double eps)
 static GEN
 max_modulus(GEN p, double tau)
 {
-  GEN q,aux,gunr;
-  long i,j,k,valuat,n=lgef(p)-3,nn,ltop=avma,bitprec,imax,e;
-  double r=0.,rho,tau2,eps;
+  GEN q,r,aux,gunr;
+  long i,j,k,valuat,n=lgef(p)-3,nn,av,ltop=avma,bitprec,M,e;
+  double rho,tau2,eps;
+
+  r = cgeti(BIGDEFAULTPREC);
+  av = avma;
 
   if (tau>3.0) tau=3.0; /* fix PZ 7.2.98: ensures eps is positive */
   eps=1/log(4./tau); tau2=tau/6.;
@@ -626,27 +633,21 @@ max_modulus(GEN p, double tau)
   aux=gdiv(gunr,(GEN) p[2+n]);
   q=gmul(aux,p); q[2+n]=lcopy(gunr);
   k=nn=n;
-  e=findpower(q); homothetie2n(q,e); r=-(double) e;
+  e=findpower(q); homothetie2n(q,e);
+  affsi(e,r);
   q=mygprec(q,bitprec+(n<<1));
   pol_to_gaussint(q,bitprec);
-  imax=(long) ((log(log(4.*n))+log(3./tau))/log(2.))+2;
+  M=(long) ((log(log(4.*n))+log(3./tau))/log(2.))+2;
   for (i=0,e=0;;)
   {
     rho=lower_bound(q,&k,eps);
     if (rho>exp2(-(double) e)) e = (long) -floor(log2(rho));
-    r -= e / exp2((double)i);
-    if (++i == imax) {
-      avma=ltop; 
-      return gpui(dbltor(2.),dbltor(r),DEFAULTPREC);
-    }
+    affii(shifti(addis(r,e), 1), r);
+    if (++i == M) break;
 
-    if (k<nn)
-      bitprec=(long) ((double) k* log2(1./tau2)+
-                      (double) (nn-k)*log2(1./eps)+
-                      3*log2((double) nn))+1;
-    else
-      bitprec=(long) ((double) nn* log2(1./tau2)+
-                      3.*log2((double) nn))+1;
+    bitprec=(long) ((double) k* log2(1./tau2)+
+                    (double) (nn-k)*log2(1./eps)+
+                    3*log2((double) nn))+1;
     homothetie_gauss(q,e,bitprec-(long)floor(mylog2((GEN) q[2+nn])+0.5));
     valuat=valuation(q);
     if (valuat>0)
@@ -655,10 +656,17 @@ max_modulus(GEN p, double tau)
       for (j=0; j<=nn; j++) q[2+j]=q[2+valuat+j];
     }
     set_karasquare_limit(gexpo(q));
-    q = gerepileupto(ltop, graeffe(q));
+    q = gerepileupto(av, graeffe(q));
     tau2=1.5*tau2; eps=1/log(1./tau2);
     e = findpower(q);
   }
+  if (!signe(r)) { avma = ltop; return realun(DEFAULTPREC); }
+  aux = cgetr(DEFAULTPREC);
+  affir(r,aux); r = aux; setexpo(r, expo(r) - M);
+  rho = rtodbl(r);
+  /* rho = sum e_i 2^-i */
+  avma = ltop;
+  return mpexp(dbltor(-rho * LOG2)); /* 2^-r */
 }
 
 /* return the k-th modulus (in ascending order) of p, rel. error tau*/
@@ -1307,6 +1315,66 @@ conformal_pol(GEN p, GEN a, long bitprec)
     if (i==0) return r;
     aux=gmul(pui,aux);
   }
+} 
+
+static GEN
+compute_radius(GEN* radii, GEN p, long k, double aux, double *delta)
+{   
+  long i, n = lgef(p)-3;
+  GEN rmin,rmax,p1;
+  if (k>1)
+  {
+    i=k-1; while (i>0 && !signe(radii[i])) i--;
+    rmin = pre_modulus(p,k,aux, radii[i], radii[k]);
+  } 
+  else /* k=1 */
+    rmin = min_modulus(p,aux);
+  affrr(rmin, radii[k]);
+ 
+  if (k+1<n) 
+  {   
+    i=k+2; while (i<=n && !signe(radii[i])) i++;
+    rmax = pre_modulus(p,k+1,aux, radii[k+1], radii[i]);
+  }   
+  else /* k+1=n */
+    rmax = max_modulus(p,aux);
+  affrr(rmax, radii[k+1]);
+  
+  p1 = radii[k];
+  for (i=k-1; i>=1; i--)
+  {
+    if (!signe(radii[i]) || cmprr(radii[i], p1) > 0)
+      affrr(p1, radii[i]);
+    else
+      p1 = radii[i];
+  }
+  p1 = radii[k+1];
+  for (i=k+1; i<=n; i++)
+  {
+    if (!signe(radii[i]) || cmprr(radii[i], p1) < 0)
+      affrr(p1, radii[i]);
+    else      
+      p1 = radii[i];
+  }
+  *delta = rtodbl(gmul2n(mplog(divrr(rmax,rmin)), -1));
+  if (*delta > 1.) *delta = 1.;
+  return mpsqrt(mulrr(rmin,rmax));
+}
+
+static GEN
+update_radius(GEN *radii, GEN rho, double *par, double *par2)
+{
+  GEN p1, invrho = ginv(rho);
+  long i, n = lg(radii);
+  double t, param = 0., param2 = 0.;
+  for (i=1; i<n; i++)
+  {
+    affrr(mulrr(radii[i], invrho), radii[i]);
+    p1 = ginv(subsr(1, radii[i]));
+    t = fabs(rtodbl(p1));
+    param += t; if (t > 1.) param2 += log2(t);
+  }
+  *par = param; *par2 = param2; return invrho;
 }
 
 /* apply the conformal mapping then split from U */
@@ -1314,8 +1382,8 @@ static void
 conformal_mapping(GEN p, long k, long bitprec, double aux,GEN *F,GEN *G)
 {
   long bitprec2,n=lgef(p)-3,decprec,i,ltop = avma, av;
-  GEN q,FF,GG,a,R,aux2, *gptr[3];
-  GEN rmin,rmax,rho,invrho;
+  GEN q,FF,GG,a,R, *gptr[3];
+  GEN rho,invrho;
   double delta,param,param2;
 
   bitprec2=bitprec+(long) (n*(2.*log2(2.732)+log2(1.5)))+1;
@@ -1335,61 +1403,16 @@ conformal_mapping(GEN p, long k, long bitprec, double aux,GEN *F,GEN *G)
       affrr(mpsqrt(addsr(1,p1)), radius[i]);
       avma = a;
     }
-  if (k>1)
-  {
-    i=k-1; while (i>0 && !signe(radius[i])) i--;
-    rmin = pre_modulus(q,k,aux/10., radius[i], radius[k]);
-  }
-  else
-    rmin = min_modulus(q,aux/10.);
-  affrr(rmin, radius[k]);
 
-  if (k+1<n)
-  {
-    i=k+2; while (i<=n && !signe(radius[i])) i++;
-    rmax = pre_modulus(q,k+1,aux/10., radius[k+1], radius[i]);
-  }
-  else /* k+1=n */
-    rmax = max_modulus(q,aux/10.);
-  affrr(rmax, radius[k+1]);
-
-  rho = mpsqrt(mulrr(rmin,rmax));
-  delta = rtodbl(gmul2n(mplog(divrr(rmax,rmin)), -1));
-  if (delta>1.) delta=1.;
+  rho = compute_radius(radius, q,k,aux/10., &delta);
+  invrho = update_radius(radius, rho, &param, &param2);
 
   bitprec2 += (long) (((double)n) * fabs(log2ir(rho)) + 1.);
-
-  invrho = ginv(rho);
   R=mygprec(invrho,bitprec2);
   q=scalepol(q,R,bitprec2);
   gptr[0]=&q; gptr[1]=&R; gptr[2]=&invrho;
   gerepilemany(av,gptr,3);
 
-  aux2 = radius[k];
-  for (i=k-1; i>=1; i--)
-  {
-    if (!signe(radius[i]))
-      affrr(aux2, radius[i]);
-    else
-      aux2 = radius[i];
-  }
-  aux2 = radius[k+1];
-  for (i=k+1; i<=n; i++)
-  {
-    if (!signe(radius[i]))
-      affrr(aux2, radius[i]);
-    else
-      aux2 = radius[i];
-  }
-  param=0.; param2=0.;
-  for (i=1; i<=n; i++)
-  {
-    double t;
-    affrr(mulrr(radius[i], invrho), radius[i]);
-    aux2 = ginv(subsr(1, radius[i]));
-    t = fabs(rtodbl(aux2));
-    param += t; if (t > 1.) param2 += log2(t);
-  }
   optimize_split(q,k,delta,bitprec2,&FF,&GG,param,param2);
   bitprec2 += n; R = ginv(R);
   FF=scalepol(FF,R,bitprec2);
@@ -1418,7 +1441,7 @@ split_2(GEN p, long bitprec, double thickness, GEN *F, GEN *G)
   GEN rmin,rmax,rho,invrho;
   double kappa,aux,delta,param,param2;
   long n=lgef(p)-3,i,j,k,bitprec2;
-  GEN q,FF,GG,R,aux2;
+  GEN q,FF,GG,R;
 
   radius = (GEN*) cgetg(n+1, t_VEC);
   for (i=1; i<n; i++) radius[i]=realzero(3);
@@ -1456,53 +1479,8 @@ split_2(GEN p, long bitprec, double thickness, GEN *F, GEN *G)
 
   if (step4==0)
   {
-    aux /= 10.;
-    if (k>1)
-    {
-      i=k-1; while (i>0 && !signe(radius[i])) i--;
-      rmin = pre_modulus(p,k,aux, radius[i], radius[k]);
-    }
-    else /* k=1 */
-      rmin = min_modulus(p,aux);
-    affrr(rmin, radius[k]);
-
-    if (k+1<n)
-    {
-      i=k+2; while (i<=n && !signe(radius[i])) i++;
-      rmax = pre_modulus(p,k+1,aux, radius[k+1], radius[i]);
-    }
-    else /* k+1=n */
-      rmax = max_modulus(p,aux);
-    affrr(rmax, radius[k+1]);
-
-    rho = mpsqrt(mulrr(rmin,rmax));
-    delta = rtodbl(gmul2n(mplog(divrr(rmax, rmin)), -1));
-
-    aux2 = radius[k];
-    for (i=k-1; i>=1; i--)
-    {
-      if (!signe(radius[i]))
-        affrr(aux2, radius[i]);
-      else
-        aux2 = radius[i];
-    }
-    aux2 = radius[k+1];
-    for (i=k+1; i<=n; i++)
-    {
-      if (!signe(radius[i]))
-        affrr(aux2, radius[i]);
-      else      
-        aux2 = radius[i];
-    }
-    invrho = ginv(rho);
-    param = 0.;param2 = 0.;
-    for (i=1; i<=n; i++)
-    {
-      affrr(mulrr(radius[i], invrho), radius[i]);
-      aux2 = ginv(subsr(1, radius[i]));
-      param += fabs(rtodbl(aux2));
-      if (cmprs(aux2, 1) > 0) param2 += log2ir(aux2);
-    }
+    rho = compute_radius(radius, p, k, aux/10., &delta);
+    invrho = update_radius(radius, rho, &param, &param2);
 
     bitprec2 = bitprec + (long) ((double)n * fabs(log2ir(rho)) + 1.);
     R = mygprec(invrho,bitprec2);
