@@ -1,4 +1,4 @@
-/* $Id: gp.c,v 1.61 2000/11/14 00:01:54 karim Exp $
+/* $Id: gp.c,v 1.61.2.7 2003/01/11 15:49:46 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -143,7 +143,7 @@ gp_preinit(int force)
   strcpy(prompt, dflt);
 
 #if defined(UNIX) || defined(__EMX__)
-#  ifdef __EMX__
+#  if defined(__EMX__) || defined(__CYGWIN32__)
   path = pari_strdup(".;C:/;C:/gp");
 #  else
   path = pari_strdup(".:~:~/gp");
@@ -1097,25 +1097,42 @@ user_member()
       }
 }
 
+static void
+center(char *s)
+{
+  long i, pad = term_width() - strlen(s);
+  char *u = thestring;
+
+  if (pad<0) pad=0; else pad >>= 1;
+  for (i=0; i<pad; i++) *u++ = ' ';
+  while (*s) *u++ = *s++;
+  *u++='\n'; *u=0; pariputs(thestring);
+}
+
 static void
 community()
 {
-  pariputs("The standard distribution of GP/PARI includes a reference manual, a tutorial\n\
-and a reference card and you should ask the person who installed PARI on\n\
-your system where they can be found. You can also download them\n\
-from the PARI WWW site http://www.parigp-home.de/\n\
-\n\
-Three mailing lists are devoted to PARI:\n\
+  long len = strlen(GPMISCDIR) + 1024;
+  char *s = gpmalloc(len);
+  
+  sprintf(s, "The standard distribution of GP/PARI includes a reference \
+manual, a tutorial, a reference card and quite a few examples. They should \
+have been installed in the directory '%s'. If not you should ask the person \
+who installed PARI on your system where they can be found. You can also \
+download them from the PARI WWW site 'http://www.parigp-home.de/'",
+GPMISCDIR);
+  print_text(s); free(s);
+
+  pariputs("\nThree mailing lists are devoted to PARI:\n\
   - pari-announce (moderated) to announce major version changes.\n\
   - pari-dev for everything related to the development of PARI, including\n\
     suggestions, technical questions, bug reports and patch submissions.\n\
-  - pari-users for everything else !\n\
-To subscribe, send an empty message to <list name>-subscribe@list.cr.yp.to\n\
-You can only send messages to these lists if you have subscribed !\n\
-An archive is kept at the WWW site mentioned above.\n\n\
-In case you don't want to subscribe, you can reach the authors directly by\n\
-email: pari@math.u-bordeaux.fr (answer not guaranteed).");
-}
+  - pari-users for everything else!\n");
+  print_text("\
+To subscribe, send an empty message to <listname>-subscribe@list.cr.yp.to. You \
+can only send messages to the lists you have subscribed to! An archive is kept \
+at the WWW site mentioned above. You can also reach the authors directly by \
+email: pari@math.u-bordeaux.fr (answer not guaranteed)."); }
 
 static void
 gentypes(void)
@@ -1411,8 +1428,10 @@ aide0(char *s, int flag)
   switch(EpVALENCE(ep))
   {
     case EpUSER:
-      print_user_fun(ep); if (!ep->help) return;
-      pariputs("\n\n"); long_help=0; break;
+      if (!ep->help || long_help) print_user_fun(ep);
+      if (!ep->help) return;
+      if (long_help) { pariputs("\n\n"); long_help=0; }
+      break;
 
     case EpGVAR:
     case EpVAR:
@@ -1511,19 +1530,6 @@ print_hash_list(char *s)
       print_entree(ep,n);
 }
 
-
-static void
-center(char *s)
-{
-  long i, pad = term_width() - strlen(s);
-  char *u = thestring;
-
-  if (pad<0) pad=0; else pad >>= 1;
-  for (i=0; i<pad; i++) *u++ = ' ';
-  while (*s) *u++ = *s++;
-  *u++='\n'; *u=0; pariputs(thestring);
-}
-
 static char *
 what_readline()
 {
@@ -1549,7 +1555,7 @@ static void
 gp_head()
 {
   print_version(); pariputs("\n");
-  center("Copyright (C) 2000 The PARI Group");
+  center("Copyright (C) 2002 The PARI Group");
   print_text("\nPARI/GP is free software, covered by the GNU General Public \
 License, and comes WITHOUT ANY WARRANTY WHATSOEVER");
   pariputs("\n\
@@ -1698,7 +1704,7 @@ normal_output(GEN z, long n)
   term_color(c_OUTPUT);
   init_lim_lines(thestring,lim_lines);
   gp_output(z);
-  init_lim_lines("",lim_lines);
+  init_lim_lines(NULL,lim_lines);
   term_color(c_NONE); pariputc('\n');
 }
 
@@ -1816,18 +1822,66 @@ escape(char *tch)
 #  include <pwd.h>
 #endif
 
-static int
-get_preproc_value(char *s)
+#define err_gprc(s,t,u) { fprintferr("\n"); err(talker2,s,t,u); }
+
+static ulong
+read_uint(char **s)
 {
-  if (!strncmp(s,"EMACS",5)) return under_emacs || under_texmacs;
-  if (!strncmp(s,"READL",5))
+  long v = atol(*s);
+  if (!isdigit((int)**s)) err_gprc("not an integer", *s, *s);
+  while (isdigit((int)**s)) (*s)++;
+  return v;
+}
+static ulong
+read_dot_uint(char **s)
+{
+  if (**s != '.') return 0;
+  (*s)++; return read_uint(s);
+}
+/* read a.b.c */
+static long
+read_version(char **s)
+{
+  long a, b, c;
+  a = read_uint(s);
+  b = read_dot_uint(s);
+  c = read_dot_uint(s);
+  return PARI_VERSION(a,b,c);
+}
+
+static int
+get_preproc_value(char **s)
+{
+  if (!strncmp(*s,"EMACS",5)) 
   {
+    *s += 5;
+    return under_emacs || under_texmacs;
+  }
+  if (!strncmp(*s,"READL",5))
+  {
+    *s += 5;
 #ifdef READLINE
     return 1;
 #else
     return 0;
 #endif
   }
+  if (!strncmp(*s,"VERSION",7))
+  {
+    int less = 0, orequal = 0;
+    long d;
+    *s += 7;
+    switch(**s)
+    {
+      case '<': (*s)++; less = 1; break;
+      case '>': (*s)++; less = 0; break;
+      default: return -1;
+    }
+    if (**s == '=') { (*s)++; orequal = 1; }
+    d = PARI_VERSION_CODE - read_version(s);
+    if (!d) return orequal;
+    return less? (d < 0): (d > 0);
+  }
   return -1;
 }
 
@@ -1931,7 +1985,7 @@ init_filtre(char *s)
 static int
 get_line_from_file(FILE *file, Buffer *b, char *prompt)
 {
-  int f_flag = prompt? f_REG | f_KEEPCASE: f_REG;
+  int f_flag = prompt? f_REG: f_REG| f_KEEPCASE;
   int wait_for_brace, wait_for_input;
   long len = b->len;
   char *s =  b->buf;
@@ -2001,8 +2055,6 @@ get_line_from_file(FILE *file, Buffer *b, char *prompt)
   return 1;
 }
 
-#define err_gprc(s,t,u) { fprintferr("\n"); err(talker2,s,t,u); }
-
 static char **
 gp_initrc()
 {
@@ -2035,11 +2087,10 @@ gp_initrc()
         s1 += 2;
         if (!strncmp(s1,"not",3)) { NOT = !NOT; s1 += 3; }
         if (*s1 == '!')           { NOT = !NOT; s1++; }
-        z = get_preproc_value(s1);
+        z = get_preproc_value(&s1);
 	if (z < 0) err_gprc("unknown preprocessor variable",s1,b->buf);
 	if (NOT) z = !z;
         if (!z) continue;
-        s1 += 5;
       }
       if (!strncmp(s1,"read",4))
       { /* read file */
