@@ -1,4 +1,4 @@
-/* $Id: mp.c,v 1.33 2000/11/10 16:11:15 karim Exp $
+/* $Id: mp.c,v 1.33.2.3 2002/01/29 13:46:23 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -2119,37 +2119,44 @@ resiimul(GEN x, GEN sy)
   return gerepileuptoint(av, r); /* = resii(x, y) */
 }
 
-/* x % (2^n) */
+/* x % (2^n), assuming x, n >= 0 */
 GEN
 resmod2n(GEN x, long n)
 {
-  long l,k,lx,ly;
-  GEN y, z, t;
-  
-  if (!signe(x)) return gzero;
+  long hi,l,k,lx,ly;
+  GEN z, xd, zd;
 
-  l = n % BITS_IN_LONG;
-  k = n / BITS_IN_LONG;
-  ly = l? k+3: k+2;
+  if (!signe(x) || !n) return gzero;
+
+  l = n & (BITS_IN_LONG-1);    /* n % BITS_IN_LONG */
+  k = n >> TWOPOTBITS_IN_LONG; /* n / BITS_IN_LONG */
   lx = lgefint(x);
-  if (lx < ly) return icopy(x);
-  z = cgeti(ly);
-  z[1] = evalsigne(1)|evallgefint(ly);
-  y = z + ly;
-  t = x + lx;
-  for ( ;k; k--) *--y = *--t;
-  if (l) *--y = *--t & ((1<<l) - 1);
-#if 0
-  if (!egalii(z, resii(x, shifti(gun,n))))
-    err(talker,"bug in resmod2n: n = %ld, x = %Z\n",n,x);
-#endif
+  if (lx < k+3) return icopy(x);
+
+  xd = x + (lx-k-1);
+  /* x = |_|...|#|1|...|k| : copy the last l bits of # and the last k words
+   *            ^--- initial xd  */
+  hi = *xd & ((1<<l) - 1); /* last l bits of # = top bits of result */
+  if (!hi)
+  { /* strip leading zeroes from result */
+    xd++; while (k && !*xd) { k--; xd++; }
+    if (!k) return gzero;
+    ly = k+2; xd--;
+  }
+  else
+    ly = k+3;
+
+  zd = z = cgeti(ly);
+  *++zd = evalsigne(1) | evallgefint(ly);
+  if (hi) *++zd = hi;
+  for ( ;k; k--) *++zd = *++xd;
   return z;
 }
 
 static GEN
 quicksqri(GEN a, long na)
 {
-  GEN a0,c,c0,c1;
+  GEN a0,c;
   long av,n0,n0a,i;
 
   if (na < KARATSUBA_SQRI_LIMIT) return sqrispec(a,na);
@@ -2159,14 +2166,14 @@ quicksqri(GEN a, long na)
   c = quicksqri(a,na);
   if (n0a)
   {
-    c0 = quicksqri(a0,n0a);
+    GEN t, c1, c0 = quicksqri(a0,n0a);
 #if 0
     c1 = shifti(quickmulii(a0,a, n0a,na),1);
 #else /* slower !!! */
-    a = addiispec(a0,a,n0a,na);
-    a = quicksqri(a+2,lgefint(a)-2);
+    t = addiispec(a0,a,n0a,na);
+    t = quicksqri(t+2,lgefint(t)-2);
     c1= addiispec(c0+2,c+2, lgefint(c0)-2, lgefint(c)-2);
-    c1= subiispec(a+2, c1+2, lgefint(a)-2, lgefint(c1)-2);
+    c1= subiispec(t+2, c1+2, lgefint(t)-2, lgefint(c1)-2);
 #endif
     c = addshiftw(c,c1, n0);
     c = addshiftw(c,c0, n0);
@@ -2294,12 +2301,6 @@ karamulir(GEN x, GEN y, long flag)
 #endif
 
 #ifdef LONG_IS_64BIT
-
-#if PARI_BYTE_ORDER == LITTLE_ENDIAN_64 || PARI_BYTE_ORDER == BIG_ENDIAN_64
-#else
-  error... unknown machine
-#endif
-
 GEN
 dbltor(double x)
 {
@@ -2343,16 +2344,14 @@ rtodbl(GEN x)
   return fi.f;
 }
 
-#else
+#else /* LONG_IS_64BIT */
 
-#if   PARI_BYTE_ORDER == LITTLE_ENDIAN
+#if   PARI_DOUBLE_FORMAT == 1
 #  define INDEX0 1
 #  define INDEX1 0
-#elif PARI_BYTE_ORDER == BIG_ENDIAN
+#elif PARI_DOUBLE_FORMAT == 0
 #  define INDEX0 0
 #  define INDEX1 1
-#else
-   error... unknown machine
 #endif
 
 GEN
@@ -2410,7 +2409,7 @@ rtodbl(GEN x)
   fi.i[INDEX1] = (a << (BITS_IN_LONG-expo_len)) | (b >> expo_len);
   return fi.f;
 }
-#endif
+#endif /* LONG_IS_64BIT */
 
 /* Old cgiv without reference count (which was not used anyway)
  * Should be a macro.
