@@ -1,4 +1,4 @@
-/* $Id: anal.c,v 1.45 2000/11/15 11:43:41 karim Exp $
+/* $Id: anal.c,v 1.45.2.8 2005/09/15 14:55:22 bill Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -35,6 +35,8 @@ typedef struct var_cell {
 
 #define separe(c) ((c)==';' || (c)==':')
 typedef GEN (*PFGEN)(ANYARG);
+typedef GEN (*F2GEN)(GEN,GEN);
+typedef GEN (*F1GEN)(GEN);
 
 static GEN    constante();
 static GEN    expr();
@@ -199,7 +201,9 @@ lisseq0(char *t, GEN (*f)(void))
   if (foreignExprHandler && *t == foreignExprSwitch)
     return (*foreignExprHandler)(t);
 
-  check_new_fun=NULL; skipping_fun_def=0;
+  redefine_fun = NULL;
+  check_new_fun = NULL;
+  skipping_fun_def = 0;
   mark.start = analyseur = t;
 
   br_status = br_NONE;
@@ -256,6 +260,10 @@ install(void *f, char *name, char *code)
   if (ep) err(warner,"[install] '%s' already there. Not replaced", name);
   else
   {
+    char *s = name;
+    if (isalpha((int)*s))
+      while (is_keyword_char(*++s)) /* empty */;
+    if (*s) err(talker2,"not a valid identifier", s, name);
     ep = installep(f, name, strlen(name), EpINSTALL, 0, functions_hash + hash);
     ep->code = pari_strdup(code);
   }
@@ -466,105 +474,104 @@ gshift_l(GEN x, GEN n)  { return gshift(x, itos(n)); }
 static GEN
 gshift_r(GEN x, GEN n) { return gshift(x,-itos(n)); }
 
+#define UNDEF (GEN)0x1
 static GEN
 expr(void)
 {
-  PFGEN f[] = { NULL,NULL,NULL,NULL };
+  long av = avma, lim = stack_lim(av, 2);
   GEN aux,e,e1,e2,e3;
-  long av = avma, lim = stack_lim(av,2);
-
-  e1 = e2 = e3 = NULL;
+  F2GEN F1,F2,F3;
+  int F0 = 0;
 
+  F1 = F2 = F3 = (F2GEN)NULL;
+  e1 = e2 = e3 = UNDEF;
 L3:
   aux = facteur();
   if (br_status) return NULL;
-  e3 = f[3]? f[3](e3,aux): aux;
+  e3 = F3? F3(e3,aux): aux;
   switch(*analyseur)
   {
-    case '*': analyseur++; f[3] = (PFGEN)&gmul; goto L3;
-    case '/': analyseur++; f[3] = (PFGEN)&gdiv; goto L3;
-    case '%': analyseur++; f[3] = (PFGEN)&gmod; goto L3;
+    case '*': analyseur++; F3 = &gmul; goto L3;
+    case '/': analyseur++; F3 = &gdiv; goto L3;
+    case '%': analyseur++; F3 = &gmod; goto L3;
     case '\\':
-      if (*++analyseur == '/') { analyseur++; f[3]=(PFGEN)&gdivround; goto L3; }
-      f[3] = (PFGEN)&gdivent; goto L3;
+      if (analyseur[1] != '/') { analyseur++; F3 = &gdivent; goto L3; }
+      analyseur += 2; F3=&gdivround; goto L3;
 
-    case '<': case '>':
-      if (analyseur[1] == *analyseur)
-      {
-        f[3] = (*analyseur == '<')? (PFGEN)&gshift_l
-                                  : (PFGEN)&gshift_r;
-        analyseur += 2; goto L3;
-      }
+    case '<':
+      if (analyseur[1] != '<') break;
+      analyseur += 2; F3 = &gshift_l; goto L3;
+    case '>':
+      if (analyseur[1] != '>') break;
+      analyseur += 2; F3 = &gshift_r; goto L3;
   }
-  f[3] = NULL;
+  F3 = (F2GEN)NULL;
 
 L2:
-  if (!e3) goto L3;
-  e2 = f[2]? f[2](e2,e3): e3;
-  e3 = NULL;
+  if (e3 == UNDEF) goto L3;
+  e2 = F2? F2(e2,e3): e3;
+  e3 = UNDEF;
   if (low_stack(lim, stack_lim(av,2)))
   {
-    GEN *gptr[2];
-    int n = 1; gptr[0]=&e2;
-    if (e1) gptr[n++]=&e1;
+    GEN *gptr[2]; gptr[0]=&e2; gptr[1]=&e1;
     if(DEBUGMEM>1) err(warnmem,"expr");
-    gerepilemany(av,gptr,n);
+    gerepilemany(av,gptr,(e1==UNDEF)?1: 2);
   }
 
   switch(*analyseur)
   {
-    case '+': analyseur++; f[2]=(PFGEN)&gadd; goto L3;
-    case '-': analyseur++; f[2]=(PFGEN)&gsub; goto L3;
+    case '+': analyseur++; F2=&gadd; goto L3;
+    case '-': analyseur++; F2=&gsub; goto L3;
   }
-  f[2] = NULL;
+  F2 = (F2GEN)NULL;
 
 L1:
-  if (!e2) goto L2;
-  e1 = f[1]? f[1](e1,e2): e2;
-  e2 = NULL;
+  if (e2 == UNDEF) goto L2;
+  e1 = F1? F1(e1,e2): e2;
+  e2 = UNDEF;
   switch(*analyseur)
   {
     case '<':
       switch(*++analyseur)
       {
-        case '=': analyseur++; f[1]=(PFGEN)&gle; goto L2;
-        case '>': analyseur++; f[1]=(PFGEN)&gne; goto L2;
+        case '=': analyseur++; F1=&gle; goto L2;
+        case '>': analyseur++; F1=&gne; goto L2;
       }
-      f[1]=(PFGEN)&glt; goto L2;
+      F1=&glt; goto L2;
 
     case '>':
-      if (*++analyseur == '=') { analyseur++; f[1]=(PFGEN)&gge; goto L2; }
-      f[1]=(PFGEN)&ggt; goto L2;
+      if (*++analyseur == '=') { analyseur++; F1=&gge; goto L2; }
+      F1=&ggt; goto L2;
 
     case '=':
-      if (analyseur[1] == '=') { analyseur+=2; f[1]=(PFGEN)&geq; goto L2; }
+      if (analyseur[1] == '=') { analyseur+=2; F1=&geq; goto L2; }
       goto L1;
 
     case '!':
-      if (analyseur[1] == '=') { analyseur+=2; f[1]=(PFGEN)&gne; goto L2; }
+      if (analyseur[1] == '=') { analyseur+=2; F1=&gne; goto L2; }
       goto L1;
   }
-  f[1] = NULL;
+  F1 = (F2GEN)NULL;
 
 /* L0: */
-  if (!e1) goto L1;
-  e = f[0]? (gcmp0(e1)? gzero: gun): e1;
-  e1 = NULL;
+  if (e1 == UNDEF) goto L1;
+  e = F0? (gcmp0(e1)? gzero: gun): e1;
+  e1 = UNDEF;
   switch(*analyseur)
   {
     case '&':
       if (*++analyseur == '&') analyseur++;
       if (gcmp0(e)) { skipexpr(); return gzero; }
-      f[0]=(PFGEN)1L; goto L1;
+      F0=1; goto L1;
 
     case '|':
       if (*++analyseur == '|') analyseur++;
       if (!gcmp0(e)) { skipexpr(); return gun; }
-      f[0]=(PFGEN)1L; goto L1;
+      F0=1; goto L1;
   }
   return e;
 }
-
+#undef UNDEF
 /********************************************************************/
 /**                                                                **/
 /**                        CHECK FUNCTIONS                         **/
@@ -1279,9 +1286,9 @@ fix(GEN x, long l)
   GEN y;
   if (typ(x) == t_COMPLEX)
   {
-    y = gcopy(x);
-    y[1] = (long)fix((GEN)y[1],l);
-    y[2] = (long)fix((GEN)y[2],l);
+    y = cgetg(3,t_COMPLEX);
+    y[1] = (long)fix((GEN)x[1],l);
+    y[2] = (long)fix((GEN)x[2],l);
   }
   else 
   {
@@ -1290,52 +1297,62 @@ fix(GEN x, long l)
   return y;
 }
 
+/* Rationale: (f(2^-e) - f(-2^-e) + O(2^-pr)) / (2 * 2^-e) = f'(0) + O(2^-2e)
+ * since 2nd derivatives cancel. 
+ *   prec(LHS) = pr - e
+ *   prec(RHS) = 2e, equal when  pr = 3e = 3/2 fpr (fpr = required final prec)
+ *
+ * For f'(x), x far from 0: prec(LHS) = pr - e - expo(x)
+ * --> pr = 3/2 fpr + expo(x) */
 static GEN
 num_deriv(void *call, GEN argvec[])
 {
-  GEN eps,a,b, x = argvec[0];
-  long pr,pr2,l,e,ex, av = avma;
+  GEN eps,a,b, y, x = argvec[0];
+  long fpr,pr,l,e,ex, av = avma;
   if (!is_const_t(typ(x))) 
   {
     a = do_call(call, x, argvec);
     return gerepileupto(av, deriv(a,gvar9(a)));
   }
-  pr = precision(x); ex = gexpo(x);
-  if (!pr) pr = prec;
-  pr2 = (long)ceil(pr * 1.5);
-  l = 2+pr2;
-  e = pr2 * BITS_IN_HALFULONG;
-  if (ex == -HIGHEXPOBIT) ex = 0;
+  fpr = precision(x)-2; /* required final prec (in sig. words) */
+  if (fpr == -2) fpr = prec-2;
+  ex = gexpo(x);
+  if (ex < 0) ex = 0; /* at 0 */
+  pr = (long)ceil(fpr * 1.5 + (ex / BITS_IN_LONG));
+  l = 2+pr;
+  e = fpr * BITS_IN_HALFULONG; /* 1/2 required prec (in sig. bits) */
 
-  eps = realun(l); setexpo(eps, ex-e);
-  x = fix(x, l);           a = do_call(call, x, argvec);
-  x = fix(gadd(x, eps),l); b = do_call(call, x, argvec);
-  setexpo(eps, e-ex);
+  eps = realun(l); setexpo(eps, -e);
+  y = fix(gsub(x, eps), l); a = do_call(call, y, argvec);
+  y = fix(gadd(x, eps), l); b = do_call(call, y, argvec);
+  setexpo(eps, e-1);
   return gerepileupto(av, gmul(gsub(b,a), eps));
 }
 
+/* as above, for user functions */
 static GEN
 num_derivU(GEN p, GEN *arg, GEN *loc, int narg, int nloc)
 {
   GEN eps,a,b, x = *arg;
-  long pr,pr2,l,e,ex, av = avma;
+  long fpr,pr,l,e,ex, av = avma;
 
   if (!is_const_t(typ(x))) 
   {
     a = call_fun(p,arg,loc,narg,nloc);
     return gerepileupto(av, deriv(a,gvar9(a)));
   }
-  pr = precision(x); ex = gexpo(x);
-  if (!pr) pr = prec;
-  pr2 = (long)ceil(pr * 1.5);
-  l = 2+pr2;
-  e = pr2 * BITS_IN_HALFULONG;
-  if (ex == -HIGHEXPOBIT) ex = 0;
+  fpr = precision(x)-2; /* required final prec (in sig. words) */
+  if (fpr == -2) fpr = prec-2;
+  ex = gexpo(x);
+  if (ex < 0) ex = 0; /* at 0 */
+  pr = (long)ceil(fpr * 1.5 + (ex / BITS_IN_LONG));
+  l = 2+pr;
+  e = fpr * BITS_IN_HALFULONG; /* 1/2 required prec (in sig. bits) */
 
-  eps = realun(l); setexpo(eps, ex-e);
-  *arg = x = fix(x,l);       a = call_fun(p,arg,loc,narg,nloc);
-  *arg = fix(gadd(x,eps),l); b = call_fun(p,arg,loc,narg,nloc);
-  setexpo(eps, e-ex);
+  eps = realun(l); setexpo(eps, -e);
+  *arg = fix(gsub(x, eps), l); a = call_fun(p,arg,loc,narg,nloc);
+  *arg = fix(gadd(x, eps), l); b = call_fun(p,arg,loc,narg,nloc);
+  setexpo(eps, e-1);
   return gerepileupto(av, gmul(gsub(b,a), eps));
 }
 
@@ -1413,7 +1430,8 @@ identifier(void)
     }
     /* return type */
     if      (*s == 'v') { ret = RET_VOID; s++; }
-    else if (*s == 'l') { ret = RET_INT;  s++; }
+    else if (*s == 'l') { ret = RET_LONG; s++; }
+    else if (*s == 'i') { ret = RET_INT;  s++; }
     else                  ret = RET_GEN;
     /* Optimized for G and p. */
     i = 0;
@@ -1574,6 +1592,11 @@ identifier(void)
 	break;
 
       case RET_INT:
+	m = (long) ((int (*)(ANYARG))call)(argvec[0], argvec[1], argvec[2], argvec[3],
+		  argvec[4], argvec[5], argvec[6], argvec[7], argvec[8]);
+	res = stoi(m); break;
+
+      case RET_LONG:
 	m = ((long (*)(ANYARG))call)(argvec[0], argvec[1], argvec[2], argvec[3],
 		  argvec[4], argvec[5], argvec[6], argvec[7], argvec[8]);
 	res = stoi(m); break;
@@ -2136,12 +2159,25 @@ static void
 doskipseq(char *c, int strict)
 {
   char *olds = analyseur;
-
+  
   mark.start = c; analyseur = c; skipseq();
   if (*analyseur)
   {
+    char *s; 
+    long L,n;
     if (strict) err(talker2,"unused characters", analyseur, c);
-    err(warner, "unused characters: %s", analyseur);
+    L = term_width();
+    n = 2 * L - (17+19+1); /* Warning + unused... + . */
+    if (strlen(analyseur) > n)
+    {
+      s = gpmalloc(n + 1);
+      n -= 5;
+      (void)strncpy(s,analyseur, n);
+      s[n] = 0; strcat(s,"[+++]");
+    }
+    else s = pari_strdup(analyseur);
+    err(warner, "unused characters: %s", s);
+    free(s);
   }
   analyseur = olds;
 }
@@ -2451,7 +2487,7 @@ skipidentifier(void)
       case '&': match_comma(); match('&'); check_var(); break;
       case 'V': match_comma(); check_var(); break;
 
-      case 'p': case 'P': case 'l': case 'v': case 'f': case 'x':
+      case 'p': case 'P': case 'l': case 'i': case 'v': case 'f': case 'x':
         break;
 
       case 'D':
@@ -3121,7 +3157,7 @@ tate(GEN x)
 static GEN
 w(GEN x)
 {
-  if (!is_bigell(x)) err(member,"tate",mark.member,mark.start);
+  if (!is_bigell(x)) err(member,"w",mark.member,mark.start);
   if (!gcmp0((GEN)x[19])) err(talker,"curve not defined over a p-adic field");
   return (GEN)x[18];
 }
@@ -3204,7 +3240,11 @@ read_member(GEN x)
     if (EpVALENCE(ep) == EpMEMBER)
       return call_fun((GEN)ep->value, NULL, &x, 0, 1);
     else
-      return ((GEN (*)(ANYARG))ep->value)(x);
+    {
+      GEN y = ((GEN (*)(ANYARG))ep->value)(x);
+      if (isonstack(y)) y = gcopy(y);
+      return y;
+    }
   }
   if (*analyseur != '=' || analyseur[1] == '=')
     err(talker2,"unknown member function",mark.member,mark.start);
@@ -3239,7 +3279,9 @@ did_break() { return br_status; }
 GEN
 return0(GEN x)
 {
+  GEN y = br_res;
   br_res = x? gclone(x): NULL;
+  if (y) gunclone(y);
   br_status = br_RETURN; return NULL;
 }
 
