@@ -1,4 +1,4 @@
-/* $Id: compat.c,v 1.7 2000/11/03 21:00:27 karim Exp $
+/* $Id: compat.c,v 1.7.2.4 2005/09/15 14:55:22 bill Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -43,7 +43,7 @@ rounderror(GEN x)
 }
 
 static void
-suppressed(ANYARG) {err(talker,"this function has been suppressed");}
+suppressed(void) {err(talker,"this function has been suppressed");}
 
 entree oldfonctions[]={
 {"O",50,NULL,7,NULL},
@@ -291,7 +291,7 @@ entree oldfonctions[]={
 {"isirreducible",1,(void *)gisirreducible,7,"G"},
 {"isisom",2,(void *)nfisisom,6,"GG"},
 {"isisomfast",2,(void *)nfisisom,6,"GG"},
-{"isoncurve",20,(void *)oncurve,5,"lGG"},
+{"isoncurve",20,(void *)oncurve,5,"iGG"},
 {"isprime",1,(void *)gisprime,4,"GD0,L,"},
 {"isprincipal",2,(void *)isprincipal,6,"GG"},
 {"isprincipalforce",2,(void *)isprincipalforce,6,"GG"},
@@ -323,7 +323,7 @@ entree oldfonctions[]={
 {"lcm",2,(void *)glcm,4,"GG"},
 {"legendre",11,(void *)legendre,7,"LDn"},
 {"length",1,(void *)glength,2,"lG"},
-{"lex",20,(void *)lexcmp,2,"lGG"},
+{"lex",20,(void *)lexcmp,2,"iGG"},
 {"lexsort",1,(void *)lexsort,8,"G"},
 {"lift",1,(void *)lift,2,"G"},
 {"lindep",1,(void *)lindep,8,"Gp"},
@@ -487,14 +487,14 @@ entree oldfonctions[]={
 {"shiftmul",21,(void *)gmul2n,1,"GL"},
 {"sigma",1,(void *)sumdiv,4,"G"},
 {"sigmak",24,(void *)sigmak0,4,"LG"},
-{"sign",10,(void *)gsigne,1,"lG"},
+{"sign",10,(void *)gsigne,1,"iG"},
 {"signat",1,(void *)signat,8,"G"},
 {"signunit",1,(void *)signunits,6,"G"},
 {"simplefactmod",2,(void *)simplefactmod,4,"GG"},
 {"simplify",1,(void *)simplify,2,"G"},
 {"sin",1,(void *)gsin,3,"Gp"},
 {"sinh",1,(void *)gsh,3,"Gp"},
-{"size",10,(void *)gsize,2,"lG"},
+{"size",10,(void *)sizedigit,2,"lG"},
 {"smallbasis",13,(void *)smallbase,6,"Gf"},
 {"smallbuchinit",93,(void *)smallbuchinit,6,"GD0.3,G,D0.3,G,D5,G,D1,G,D4,L,D3,L,p"},
 {"smalldiscf",1,(void *)smalldiscf,6,"G"},
@@ -532,7 +532,7 @@ entree oldfonctions[]={
 {"tchebi",11,(void *)tchebi,7,"LDn"},
 {"teich",1,(void *)teich,3,"Gp"},
 {"theta",2,(void *)theta,3,"GGp"},
-{"thetanullk",21,(void *)thetanullk,3,"GL"},
+{"thetanullk",99,(void *)thetanullk,3,"GLp"},
 {"threetotwo",4,(void *)threetotwo,6,"GGGG"},
 {"threetotwo2",4,(void *)threetotwo2,6,"GGGG"},
 {"torsell",1,(void *)torsell,5,"Gp"},
