@@ -1,4 +1,4 @@
-/* $Id: init.c,v 1.68 2000/11/14 12:08:46 karim Exp $
+/* $Id: init.c,v 1.68.2.5 2005/09/15 14:55:22 bill Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -572,12 +572,32 @@ newbloc(long n)
   return cur_bloc = x;
 }
 
-void
-killbloc0(GEN x, int inspect)
+/* recursively look for clones in the container and kill them */
+static void
+inspect(GEN x)
 {
-  long tx,lx,l,i,j;
-  GEN p1;
+  long i, lx;
+  switch(typ(x)) /* HACK: if x is not a GEN, we have typ(x)=0 */
+  {
+    case t_VEC: case t_COL: case t_MAT:
+      lx = lg(x);
+      for (i=1;i<lx;i++) inspect((GEN)x[i]);
+      break;
+    case t_LIST:
+      lx = lgef(x);
+      for (i=2;i<lx;i++) inspect((GEN)x[i]);
+      break;
+  }
+  if (isclone(x)) gunclone(x); /* Don't inspect here! components are dead */
+}
 
+/* If insp is set, recursively inspect x, killing all clones found. The GP
+ * expression x[i] = y is implemented as x[i] := gclone(y) and we need to
+ * reclaim the memory. Useless to inspect when x does not correspond to a GP
+ * variable [not dangerous, though] */
+void
+killbloc0(GEN x, int insp)
+{
   if (!x || isonstack(x)) return;
   if (bl_next(x)) bl_prev(bl_next(x)) = bl_prev(x);
   else 
@@ -588,45 +608,10 @@ killbloc0(GEN x, int inspect)
   if (bl_prev(x)) bl_next(bl_prev(x)) = bl_next(x);
   if (DEBUGMEM > 2)
     fprintferr("killing bloc (no %ld): %08lx\n", bl_num(x), x);
-  if (inspect)
-  {
-    /* FIXME: SIGINT should be blocked at this point */
-    tx=typ(x); /* if x is not a GEN, we will have tx=0 */
-    if (is_vec_t(tx))
-    {
-      lx = lg(x);
-      for (i=1;i<lx;i++)
-      {
-        p1=(GEN)x[i];
-        if (isclone(p1)) killbloc(p1);
-      }
-    }
-    else if (tx==t_MAT)
-    {
-      lx = lg(x);
-      if (lx>1)
-      {
-        l=lg(x[1]);
-        if (l>1)
-          for (i=1;i<lx;i++)
-            for (j=1;j<l;j++)
-            {
-              p1=gmael(x,i,j);
-              if (isclone(p1)) killbloc(p1);
-            }
-      }
-    }
-    else if (tx==t_LIST)
-    {
-      lx = lgef(x);
-      for (i=2;i<lx;i++)
-      {
-        p1=(GEN)x[i];
-        if (isclone(p1)) killbloc(p1);
-      }
-    }
-    unsetisclone(x);
-    /* FIXME: SIGINT should be released here */
+  if (insp)
+  { /* FIXME: SIGINT should be blocked until inspect() returns */
+    unsetisclone(x); /* important: oo recursion otherwise */
+    inspect(x);
   }
   free((void *)bl_base(x));
 }
@@ -1460,6 +1445,7 @@ checkmemory(GEN z)
 
 # include <sys/times.h>
 # include <sys/time.h>
+# include <time.h>
   static long
   timer_proto(int i)
   {
@@ -1749,7 +1735,7 @@ entree functions_basic[]={
 {"bnfreg",99,(void*)regulator,6,"GDGp"},
 {"bnfsignunit",18,(void*)signunits,6,"G"},
 {"bnfsunit",99,(void*)bnfsunit,6,"GGp"},
-{"bnfunit",1,(void*)buchfu,6,"Gp"},
+{"bnfunit",18,(void*)buchfu,6,"G"},
 {"bnrL1",99,(void*)bnrL1,6,"GGD0,L,p"},
 {"bnrclass",99,(void*)bnrclass0,6,"GGD0,L,p"},
 {"bnrclassno",2,(void*)rayclassno,6,"GG"},
@@ -1804,19 +1790,19 @@ entree functions_basic[]={
 {"ellheight",99,(void*)ellheight0,5,"GGD0,L,p"},
 {"ellheightmatrix",29,(void*)mathell,5,"GGp"},
 {"ellinit",99,(void*)ellinit0,5,"GD0,L,p"},
-{"ellisoncurve",20,(void*)oncurve,5,"lGG"},
+{"ellisoncurve",20,(void*)oncurve,5,"iGG"},
 {"ellj",1,(void*)jell,5,"Gp"},
 {"elllocalred",2,(void*)localreduction,5,"GG"},
 {"elllseries",99,(void*)lseriesell,5,"GGDGp"},
 {"ellorder",2,(void*)orderell,5,"GG"},
 {"ellordinate",29,(void*)ordell,5,"GGp"},
 {"ellpointtoz",29,(void*)zell,5,"GGp"},
-{"ellpow",99,(void*)powell,5,"GGGp"},
+{"ellpow",99,(void*)powell,5,"GGG"},
 {"ellrootno",99,(void*)ellrootno,5,"lGDG"},
 {"ellsigma",99,(void*)ellsigma,5,"GGD0,L,p"},
-{"ellsub",99,(void*)subell,5,"GGGp"},
-{"elltaniyama",1,(void*)taniyama,5,"Gp"},
-{"elltors",99,(void*)elltors0,5,"GD0,L,p"},
+{"ellsub",99,(void*)subell,5,"GGG"},
+{"elltaniyama",18,(void*)taniyama,5,"G"},
+{"elltors",99,(void*)elltors0,5,"GD0,L,"},
 {"ellwp",99,(void*)ellwp0,5,"GDGD0,L,pP"},
 {"ellzeta",99,(void*)ellzeta,5,"GGp"},
 {"ellztopoint",29,(void*)pointell,5,"GGp"},
@@ -1845,7 +1831,7 @@ entree functions_basic[]={
 {"forvec",87,(void*)forvec,11,"vV=GID0,L,"},
 {"frac",18,(void*)gfrac,2,"G"},
 {"galoisfixedfield",99,(void*)galoisfixedfield,6,"GGD0,L,Dn"},
-{"galoisinit",99,(void*)galoisinit,6,"GDG,"},
+{"galoisinit",99,(void*)galoisinit,6,"GDG"},
 {"galoispermtopol",2,(void*)galoispermtopol,6,"GG"},
 {"galoissubcyclo",99,(void*)galoissubcyclo,6,"LGDGDn"},
 {"gamma",1,(void*)ggamma,3,"Gp"},
@@ -1869,7 +1855,7 @@ entree functions_basic[]={
 {"idealinv",25,(void*)idealinv0,6,"GGD0,L,"},
 {"ideallist",99,(void*)ideallist0,6,"GLD4,L,"},
 {"ideallistarch",99,(void*)ideallistarch0,6,"GGDGD0,L,"},
-{"ideallog",99,(void*)zideallog,6,"GGGp"},
+{"ideallog",3,(void*)zideallog,6,"GGG"},
 {"idealmin",99,(void*)minideal,6,"GGDGp"},
 {"idealmul",99,(void*)idealmul0,6,"GGGD0,L,p"},
 {"idealnorm",2,(void*)idealnorm,6,"GG"},
@@ -1895,7 +1881,7 @@ entree functions_basic[]={
 {"kronecker",2,(void*)gkronecker,4,"GG"},
 {"lcm",2,(void*)glcm,4,"GG"},
 {"length",10,(void*)glength,2,"lG"},
-{"lex",20,(void*)lexcmp,1,"lGG"},
+{"lex",20,(void*)lexcmp,1,"iGG"},
 {"lift",99,(void*)lift0,2,"GDn"},
 {"lindep",99,(void*)lindep0,8,"GD0,L,p"},
 {"listcreate",11,(void*)listcreate,8,"L"},
@@ -1925,7 +1911,7 @@ entree functions_basic[]={
 {"matintersect",2,(void*)intersect,8,"GG"},
 {"matinverseimage",2,(void*)inverseimage,8,"GG"},
 {"matisdiagonal",10,(void*)isdiagonal,8,"lG"},
-{"matker",99,(void*)matker0,8,"GD0,L,p"},
+{"matker",99,(void*)matker0,8,"GD0,L,"},
 {"matkerint",99,(void*)matkerint0,8,"GD0,L,"},
 {"matmuldiagonal",2,(void*)matmuldiagonal,8,"GG"},
 {"matmultodiagonal",2,(void*)matmultodiagonal,8,"GG"},
@@ -1937,7 +1923,7 @@ entree functions_basic[]={
 {"matsnf",99,(void*)matsnf0,8,"GD0,L,"},
 {"matsolve",2,(void*)gauss,8,"GG"},
 {"matsolvemod",99,(void*)matsolvemod0,8,"GGGD0,L,"},
-{"matsupplement",1,(void*)suppl,8,"Gp"},
+{"matsupplement",18,(void*)suppl,8,"G"},
 {"mattranspose",18,(void*)gtrans,8,"G"},
 {"max",2,(void*)gmax,1,"GG"},
 {"min",2,(void*)gmin,1,"GG"},
@@ -1978,7 +1964,7 @@ entree functions_basic[]={
 {"nfmodprinit",2,(void*)nfmodprinit,6,"GG"},
 {"nfnewprec",1,(void*)nfnewprec,6,"Gp"},
 {"nfroots",99,(void*)nfroots,6,"GG"},
-{"nfrootsof1",1,(void*)rootsof1,6,"Gp"},
+{"nfrootsof1",18,(void*)rootsof1,6,"G"},
 {"nfsnf",2,(void*)nfsmith,6,"GG"},
 {"nfsolvemodpr",4,(void*)nfsolvemodpr,6,"GGGG"},
 {"nfsubfields",99,(void*)subfields0,6,"GDG"},
@@ -2013,7 +1999,7 @@ entree functions_basic[]={
 {"polrootspadic",32,(void*)rootpadic,7,"GGL"},
 {"polsturm",99,(void*)sturmpart,7,"lGDGDG"},
 {"polsubcyclo",99,(void*)subcyclo,7,"GGDn"},
-{"polsylvestermatrix",29,(void*)sylvestermatrix,7,"GGp"},
+{"polsylvestermatrix",2,(void*)sylvestermatrix,7,"GG"},
 {"polsym",21,(void*)polsym,7,"GL"},
 {"poltchebi",99,(void*)tchebi,7,"LDn"},
 {"poltschirnhaus",18,(void*)tschirnhaus,6,"G"},
@@ -2059,7 +2045,7 @@ entree functions_basic[]={
 {"rnfbasis",2,(void*)rnfbasis,6,"GG"},
 {"rnfbasistoalg",2,(void*)rnfbasistoalg,6,"GG"},
 {"rnfcharpoly",99,(void*)rnfcharpoly,6,"GGGDn"},
-{"rnfconductor",29,(void*)rnfconductor,6,"GGp"},
+{"rnfconductor",29,(void*)rnfconductor,6,"GGD0,L,p"},
 {"rnfdedekind",99,(void*)rnfdedekind,6,"GGG"},
 {"rnfdet",99,(void*)rnfdet0,6,"GGDG"},
 {"rnfdisc",2,(void*)rnfdiscf,6,"GG"},
@@ -2072,7 +2058,7 @@ entree functions_basic[]={
 {"rnfidealabstorel",2,(void*)rnfidealabstorel,6,"GG"},
 {"rnfidealdown",2,(void*)rnfidealdown,6,"GG"},
 {"rnfidealhnf",2,(void*)rnfidealhermite,6,"GG"},
-{"rnfidealmul",2,(void*)rnfidealmul,6,"GG"},
+{"rnfidealmul",3,(void*)rnfidealmul,6,"GGG"},
 {"rnfidealnormabs",2,(void*)rnfidealnormabs,6,"GG"},
 {"rnfidealnormrel",2,(void*)rnfidealnormrel,6,"GG"},
 {"rnfidealreltoabs",2,(void*)rnfidealreltoabs,6,"GG"},
@@ -2101,16 +2087,16 @@ entree functions_basic[]={
 {"shift",99,(void*)gshift,1,"GL"},
 {"shiftmul",99,(void*)gmul2n,1,"GL"},
 {"sigma",99,(void*)gsumdivk,4,"GD1,L,"},
-{"sign",10,(void*)gsigne,1,"lG"},
+{"sign",10,(void*)gsigne,1,"iG"},
 {"simplify",18,(void*)simplify,2,"G"},
 {"sin",1,(void*)gsin,3,"Gp"},
 {"sinh",1,(void*)gsh,3,"Gp"},
 {"sizebyte",10,(void*)taille2,2,"lG"},
-{"sizedigit",10,(void*)gsize,2,"lG"},
+{"sizedigit",10,(void*)sizedigit,2,"lG"},
 {"solve",37,(void*)zbrent,9,"V=GGEp"},
 {"sqr",18,(void*)gsqr,3,"G"},
 {"sqrt",1,(void*)gsqrt,3,"Gp"},
-{"sqrtint",1,(void*)racine,4,"G"},
+{"sqrtint",18,(void*)racine,4,"G"},
 {"sqrtn",99,(void*)gsqrtn,3,"GGD&p"},
 {"subgrouplist",99,(void*)subgrouplist0,6,"GD0,L,D0,L,p"},
 {"subst",26,(void*)gsubst,7,"GnG"},
@@ -2122,19 +2108,19 @@ entree functions_basic[]={
 {"tan",1,(void*)gtan,3,"Gp"},
 {"tanh",1,(void*)gth,3,"Gp"},
 {"taylor",12,(void*)tayl,7,"GnP"},
-{"teichmuller",1,(void*)teich,3,"Gp"},
+{"teichmuller",18,(void*)teich,3,"G"},
 {"theta",29,(void*)theta,3,"GGp"},
 {"thetanullk",99,(void*)thetanullk,3,"GLp"},
 {"thue",99,(void*)thue,7,"GGDG"},
 {"thueinit",99,(void*)thueinit,7,"GD0,L,p"},
-{"trace",1,(void*)gtrace,8,"Gp"},
+{"trace",18,(void*)gtrace,8,"G"},
 {"truncate",99,(void*)trunc0,2,"GD&"},
 {"until",82,NULL,11,NULL},
 {"valuation",20,(void*)ggval,2,"lGG"},
 {"variable",18,(void*)gpolvar,2,"G"},
 {"vecextract",99,(void*)extract0,8,"GGDG"},
-{"vecmax",1,(void*)vecmax,1,"Gp"},
-{"vecmin",1,(void*)vecmin,1,"Gp"},
+{"vecmax",18,(void*)vecmax,1,"G"},
+{"vecmin",18,(void*)vecmin,1,"G"},
 {"vecsort",99,(void*)vecsort0,8,"GDGD0,L,"},
 {"vector",28,(void*)vecteur,8,"GDVDI"},
 {"vectorv",28,(void*)vvecteur,8,"GDVDI"},
@@ -2146,7 +2132,7 @@ entree functions_basic[]={
 {"znlog",2,(void*)znlog,4,"GG"},
 {"znorder",18,(void*)order,4,"G"},
 {"znprimroot",18,(void*)ggener,4,"G"},
-{"znstar",1,(void*)znstar,4,"Gp"},
+{"znstar",18,(void*)znstar,4,"G"},
 
 /* DO NOT REMOVE THIS BLANK LINE: chname & helpsynchro depend on it */
 {NULL,0,NULL,0,NULL} /* sentinel */
