@@ -1,4 +1,4 @@
-/* $Id: elliptic.c,v 1.20 2000/11/03 21:00:28 karim Exp $
+/* $Id: elliptic.c,v 1.20.2.7 2002/09/12 01:02:58 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -766,13 +766,8 @@ new_coords(GEN e, GEN x, GEN *pta, GEN *ptb, long prec)
   b = gmul2n(w,-1);
   r1 = gsub(a,b);
   p1 = gadd(x, gmul2n(gadd(e1,r0),-1));
-  if (gcmp0(p1))
-    p1 = gsqrt(gneg_i(gmul(a,r1)),prec);
-  else
-  {
-    GEN delta = gdiv(gmul(a,r1),gsqr(p1));
-    p1 = gmul2n(gmul(p1,gaddsg(1,gsqrt(gsubsg(1,gmul2n(delta,2)),prec))),-1);
-  }
+  p1 = gmul2n(p1,-1);
+  p1 = gadd(p1, gsqrt(gsub(gsqr(p1), gmul(a,r1)), prec));
   *pta = a; *ptb = b;
   return gmul(p1,gsqr(gmul2n(gaddsg(1,gsqrt(gdiv(gadd(p1,r1),p1),prec)),-1)));
 }
@@ -811,6 +806,7 @@ zell(GEN e, GEN z, long prec)
     if (gsigne(greal(b)) != sw) b = gneg_i(b);
     a = gmul2n(gadd(gadd(a0,b0),gmul2n(b,1)),-2);
     r1 = gsub(a,b);
+    if (gcmp0(r1) || gexpo(r1) < gexpo(a) - bit_accuracy(prec)) break;
     p1 = gsqrt(gdiv(gadd(x0,r1),x0),prec);
     x1 = gmul(x0,gsqr(gmul2n(gaddsg(1,p1),-1)));
     r1 = gsub(x1,x0);
@@ -1341,14 +1337,15 @@ multi_invmod(GEN x, GEN p)
 static GEN
 addsell(GEN e, GEN z1, GEN z2, GEN p)
 {
-  GEN p1,p2,x,x1,x2,y,y1,y2;
-  long av = avma;
+  GEN z,p1,p2,x,x1,x2,y,y1,y2;
+  ulong av;
 
   if (!z1) return z2;
   if (!z2) return z1;
 
   x1 = (GEN)z1[1]; y1 = (GEN)z1[2];
   x2 = (GEN)z2[1]; y2 = (GEN)z2[2];
+  z = cgetg(3, t_VEC); av = avma;
   p2 = subii(x2, x1);
   if (p2 == gzero)
   {
@@ -1360,11 +1357,12 @@ addsell(GEN e, GEN z1, GEN z2, GEN p)
   else p1 = subii(y2,y1);
   p1 = mulii(p1, mpinvmod(p2, p));
   p1 = resii(p1, p);
-  x = subii(sqri(p1), addii(x1,x2)); x = modii(x,p);
+  x = subii(sqri(p1), addii(x1,x2));
   y = negi(addii(y1, mulii(p1,subii(x,x1))));
-  avma = av; p1 = cgetg(3,t_VEC);
-  p1[1] = licopy(x);
-  p1[2] = lmodii(y, p); return p1;
+  x = modii(x,p);
+  y = modii(y,p); avma = av;
+  z[1] = licopy(x);
+  z[2] = licopy(y); return z;
 }
 
 /* z1 <-- z1 + z2 */
@@ -1390,6 +1388,15 @@ addsell_part2(GEN e, GEN z1, GEN z2, GEN p, GEN p2inv)
   affii(y, y1);
 }
 
+static GEN
+negsell(GEN f, GEN p)
+{
+  GEN g = cgetg(3, t_VEC);
+  g[1] = f[1];
+  g[2] = signe(f[2])? lsubii(p, (GEN)f[2]): f[2];
+  return g;
+}
+
 static GEN
 powsell(GEN e, GEN z, GEN n, GEN p)
 {
@@ -1400,9 +1407,8 @@ powsell(GEN e, GEN z, GEN n, GEN p)
   if (!s || !z) return NULL;
   if (s < 0)
   {
-    n = negi(n); y = cgetg(3,t_VEC);
-    y[2] = lnegi((GEN)z[2]);
-    y[1] = z[1]; z = y;
+    z = negsell(z, p);
+    n = negi(n);
   }
   if (is_pm1(n)) return z;
 
@@ -1421,6 +1427,25 @@ powsell(GEN e, GEN z, GEN n, GEN p)
   return addsell(e,y,z,p);
 }
 
+/* assume H.f = 0, return exact order of f */
+static GEN
+exact_order(GEN H, GEN f, GEN cp4, GEN p)
+{
+  GEN P, e, g, h = H, fa = decomp(H);
+  long i, j, l;
+
+  P = (GEN)fa[1]; l = lg(P);
+  e = (GEN)fa[2];
+  for (i=1; i<l; i++)
+    for (j=itos((GEN)e[i]); j; j--)
+    {
+      g = divii(h,(GEN)P[i]);
+      if (powsell(cp4,f,g,p)) break;
+      h = g;
+    }
+  return h;
+}
+
 /* make sure *x has lgefint >= k */
 static void
 _fix(GEN x, long k)
@@ -1432,51 +1457,66 @@ _fix(GEN x, long k)
 /* low word of integer x */
 #define _low(x) (__x=(GEN)x, __x[lgefint(__x)-1])
 
-/* compute a_p using Shanks/Mestre + Montgomery's trick. Assume p > 20, say */
+/* compute a_p using Shanks/Mestre + Montgomery's trick. Assume p > 457 */
 GEN
 apell1(GEN e, GEN p)
 {
-  long *tx, *ty, *ti, av = avma, av2,pfinal,i,j,j2,s,flc,flcc,x,nb;
-  GEN p1,p2,p3,h,mfh,f,fh,fg,pordmin,u,v,p1p,p2p,acon,bcon,c4,c6,cp4,pts;
+  long *tx, *ty, *ti, pfinal, i, j, s, k, k1, x, nb;
+  ulong av = avma, av2;
+  GEN p1,h,mfh,f,fh,fg,pordmin,u,v,p1p,p2p,A,B,c4,c6,cp4,pts;
   GEN __x;
-
-  if (DEBUGLEVEL) timer2();
-  p1 = gmodulsg(1,p);
-  c4 = gdivgs(gmul(p1,(GEN)e[10]), -48);
-  c6 = gdivgs(gmul(p1,(GEN)e[11]), -864);
-  pordmin = gceil(gmul2n(gsqrt(p,DEFAULTPREC),2));
-  p1p = addsi(1,p); p2p = shifti(p1p,1);
-  x=0; flcc=0; flc = kronecker((GEN)c6[2],p);
-  u=c6; acon=gzero; bcon=gun; h=p1p;
   tx = ty = ti = NULL; /* gcc -Wall */
+
+  if (DEBUGLEVEL) (void)timer2();
+  c4 = gmod(gdivgs((GEN)e[10],  -48), p);
+  c6 = gmod(gdivgs((GEN)e[11], -864), p);
+  pordmin = gceil(gmul2n(gsqrt(p,DEFAULTPREC),2)); /* ceil( sqrt(4p) ) */
+  p1p = addsi(1,p);
+  p2p = shifti(p1p,1);
+  x = 0; u = c6; k1 = 0; k = kronecker(c6, p);
+  A = gzero; B = gun; h = p1p;
   for(;;)
   {
-    while (flc==flcc || !flc)
+    while (k == k1 || !k)
     {
       x++;
-      u = gadd(c6, gmulsg(x, gaddgs(c4,x*x)));
-      flc = kronecker((GEN)u[2],p);
+      u = modii(addii(c6, mulsi(x, addii(c4, mulss(x,x)))), p);
+      k = kronecker(u, p);
     }
-    flcc = flc;
+    k1 = k;
+
     f = cgetg(3,t_VEC);
-    f[1] = (long)lift_intern(gmulsg(x,u));
-    f[2] = (long)lift_intern(gsqr(u));
-    cp4 = lift_intern(gmul(c4, (GEN)f[2]));
+    f[1] = lmodii(mulsi(x,u), p);
+    f[2] = lmodii(sqri(u),    p);
+    cp4 = modii(mulii(c4, (GEN)f[2]), p);
     fh = powsell(cp4,f,h,p);
-    s = itos(gceil(gsqrt(gdiv(pordmin,bcon),DEFAULTPREC))) >> 1;
-    nb = min(128, s >> 1);
+    s = itos(gceil(gsqrt(gdiv(pordmin,B),DEFAULTPREC))) >> 1;
     /* look for h s.t f^h = 0 */
-    if (bcon == gun)
+    if (B == gun)
     { /* first time: initialize */
-      tx = newbloc(s+1); *tx = evaltyp(t_VECSMALL) | evallg(s+1);
+      tx = newbloc(s+1);
       ty = newbloc(s+1);
       ti = newbloc(s+1);
     }
-    else f = powsell(cp4,f,bcon,p); /* F */
+    else f = powsell(cp4,f,B,p); /* F = B.f */
+    *tx = evaltyp(t_VECSMALL) | evallg(s+1);
     if (!fh) goto FOUND;
 
     p1 = gcopy(fh);
-    pts = new_chunk(nb+1);
+    if (s < 3)
+    { /* we're nearly done: naive search */
+      GEN q1 = p1, mf = negsell(f, p); /* -F */
+      for (i=1;; i++)
+      {
+        p1 = addsell(cp4,p1, f,p); /* h.f + i.F */
+        if (!p1) { h = addii(h, mulsi( i,B)); goto FOUND; }
+        q1 = addsell(cp4,q1,mf,p); /* h.f - i.F */
+        if (!q1) { h = addii(h, mulsi(-i,B)); goto FOUND; }
+      }
+    }
+    /* Baby Step/Giant Step */
+    nb = min(128, s >> 1); /* > 0. Will do nb pts at a time: faster inverse */
+    pts = cgetg(nb+1, t_VEC);
     j = lgefint(p);
     for (i=1; i<=nb; i++)
     { /* baby steps */
@@ -1484,12 +1524,11 @@ apell1(GEN e, GEN p)
       _fix(p1+1, j); tx[i] = _low((GEN)p1[1]);
       _fix(p1+2, j); ty[i] = _low((GEN)p1[2]);
       p1 = addsell(cp4,p1,f,p); /* h.f + i.F */
-      if (!p1) { h = addii(h, mulsi(i,bcon)); goto FOUND; }
+      if (!p1) { h = addii(h, mulsi(i,B)); goto FOUND; }
     }
-    mfh = dummycopy(fh);
-    mfh[2] = lnegi((GEN)mfh[2]);
-    fg = addsell(cp4,p1,mfh,p); /* nb.F */
-    if (!fg) { h = mulsi(nb,bcon); goto FOUND; }
+    mfh = negsell(fh, p);
+    fg = addsell(cp4,p1,mfh,p); /* h.f + nb.F - h.f = nb.F */
+    if (!fg) { h = mulsi(nb,B); goto FOUND; }
     u = cgetg(nb+1, t_VEC);
     av2 = avma; /* more baby steps, nb points at a time */
     while (i <= s)
@@ -1502,9 +1541,8 @@ apell1(GEN e, GEN p)
         if (u[j] == zero) /* sum = 0 or doubling */
         {
           long k = i+j-2;
-          if (egalii((GEN)p1[2],(GEN)fg[2])) k -= 2*nb; /* fg = p1 */
-          h = addii(h, mulsi(k,bcon));
-          goto FOUND;
+          if (egalii((GEN)p1[2],(GEN)fg[2])) k -= 2*nb; /* fg == p1 */
+          h = addii(h, mulsi(k,B)); goto FOUND;
         }
       }
       v = multi_invmod(u, p);
@@ -1518,12 +1556,13 @@ apell1(GEN e, GEN p)
       }
       avma = av2;
     }
-    p1 = addsell(cp4,(GEN)pts[j-1],mfh,p); /* = f^(s-1) */
+    p1 = addsell(cp4,(GEN)pts[j-1],mfh,p); /* = (s-1).F */
+    if (!p1) { h = mulsi(s-1,B); goto FOUND; }
     if (DEBUGLEVEL) msgtimer("[apell1] baby steps, s = %ld",s);
 
-    /* giant steps: fg = f^s */
+    /* giant steps: fg = s.F */
     fg = addsell(cp4,p1,f,p);
-    if (!fg) { h = addii(h, mulsi(s,bcon)); goto FOUND; }
+    if (!fg) { h = mulsi(s,B); goto FOUND; }
     pfinal = _low(p); av2 = avma;
 
     p1 = gen_sort(tx, cmp_IND | cmp_C, NULL);
@@ -1534,8 +1573,12 @@ apell1(GEN e, GEN p)
     avma = av2;
 
     gaffect(fg, (GEN)pts[1]);
-    for (j=2; j<=nb; j++) /* pts = first nb multiples of fg */
-      gaffect(addsell(cp4,(GEN)pts[j-1],fg,p), (GEN)pts[j]);
+    for (j=2; j<=nb; j++) /* pts[j] = j.fg = (s*j).F */
+    {
+      p1 = addsell(cp4,(GEN)pts[j-1],fg,p);
+      if (!p1) { h = mulii(mulss(s,j), B); goto FOUND; }
+      gaffect(p1, (GEN)pts[j]);
+    }
     /* replace fg by nb.fg since we do nb points at a time */
     avma = av2;
     fg = gcopy((GEN)pts[nb]);
@@ -1545,7 +1588,7 @@ apell1(GEN e, GEN p)
     {
       GEN ftest = (GEN)pts[j];
       ulong m, l = 1, r = s+1;
-      long k, k2;
+      long k, k2, j2;
 
       avma = av2;
       k = _low((GEN)ftest[1]);
@@ -1554,27 +1597,27 @@ apell1(GEN e, GEN p)
         m = (l+r) >> 1;
         if (tx[m] < k) l = m+1; else r = m;
       }
-      if (r <= s && tx[r] == k)
+      if (r <= (ulong)s && tx[r] == k)
       {
         while (tx[r] == k && r) r--;
         k2 = _low((GEN)ftest[2]);
-        for (r++; tx[r] == k && r <= s; r++)
+        for (r++; tx[r] == k && r <= (ulong)s; r++)
           if (ty[r] == k2 || ty[r] == pfinal - k2)
           { /* [h+j2] f ==  ftest (= [i.s] f)? */
-            if (DEBUGLEVEL) msgtimer("[apell1] giant steps, i = %ld",i);
             j2 = ti[r] - 1;
+            if (DEBUGLEVEL) msgtimer("[apell1] giant steps, i = %ld",i);
             p1 = addsell(cp4, powsell(cp4,f,stoi(j2),p),fh,p);
             if (egalii((GEN)p1[1], (GEN)ftest[1]))
             {
               if (egalii((GEN)p1[2], (GEN)ftest[2])) i = -i;
-              h = addii(h, mulii(addis(mulss(s,i), j2), bcon));
+              h = addii(h, mulii(addis(mulss(s,i), j2), B));
               goto FOUND;
             }
           }
       }
       if (++j > nb)
       { /* compute next nb points */
-        long save = 0; /* gcc -Wall */
+        long save = 0; /* gcc -Wall */;
         for (j=1; j<=nb; j++)
         {
           p1 = (GEN)pts[j];
@@ -1593,35 +1636,29 @@ apell1(GEN e, GEN p)
       }
     }
 
-FOUND: /* success, found a point of exponent h */
-    p2 = decomp(h); p1=(GEN)p2[1]; p2=(GEN)p2[2];
-    for (i=1; i<lg(p1); i++)
-      for (j=itos((GEN)p2[i]); j; j--)
-      {
-        p3 = divii(h,(GEN)p1[i]);
-        if (powsell(cp4,f,p3,p)) break;
-	h = p3;
-      }
-    /* now h is the exact order */
-    if (bcon == gun) bcon = h;
+FOUND: /* found a point of exponent h */
+    h = exact_order(h, f,cp4,p);
+    /* now h is the exact order, and divides E(Fp) = A (mod B) */
+    if (B == gun) B = h;
     else
     {
-      p1 = chinois(gmodulcp(acon,bcon), gmodulsg(0,h));
-      acon = (GEN)p1[2];
-      bcon = (GEN)p1[1];
+      p1 = chinois(gmodulcp(A,B), gmodulsg(0,h));
+      A = (GEN)p1[2];
+      B = (GEN)p1[1];
     }
 
-    i = (cmpii(bcon, pordmin) < 0);
-    if (i) acon = centermod(subii(p2p,acon), bcon);
-    p1 = ground(gdiv(gsub(p1p,acon),bcon));
-    h = addii(acon, mulii(p1,bcon));
+    i = (cmpii(B, pordmin) < 0);
+    if (i) A = centermod(subii(p2p,A), B);
+    p1 = ground(gdiv(gsub(p1p,A),B));
+    h = addii(A, mulii(p1,B));
+    /* h = A mod B, closest lift to p+1 */
     if (!i) break;
   }
   gunclone(tx);
   gunclone(ty);
   gunclone(ti);
-  p1 = (flc==1)? subii(p1p,h): subii(h,p1p);
-  return gerepileupto(av,p1);
+  p1 = (k==1)? subii(p1p,h): subii(h,p1p);
+  return gerepileuptoint(av,p1);
 }
 
 typedef struct
@@ -2856,20 +2893,22 @@ torsellnagelllutz(GEN e)
 
 /* Using Doud's algorithm */
 
-/* Input e and n, finds a bound for #Tor */
+/* finds a bound for #E_tor */
 static long
-torsbound(GEN e, long n)
+torsbound(GEN e)
 {
-  long av = avma, m, b, c, d, prime = 2;
+  long m, b, c, d, prime = 2;
+  ulong av = avma;
   byteptr p = diffptr;
   GEN D = (GEN)e[12];
-
+  long n = bit_accuracy(lgefint(D)) >> 3;
+  /* n = number of primes to try ~ 1 prime every 8 bits in D */
   b = c = m = 0; p++;
   while (m<n)
   {
     d = *p++; if (!d) err(primer1);
     prime += d;
-    if (ggval(D,stoi(prime)) == 0)
+    if (smodis(D, prime))
     {
       b = cgcd(b, prime+1 - itos(apell0(e,prime)));
       if (b==c) m++; else {c = b; m = 0;}
@@ -2888,8 +2927,8 @@ _round(GEN x, long *e)
   return y;
 }
 
-/* Input the curve, a point, and an integer n, returns a point of order n
-   on the curve, or NULL if q is not rational. */
+/* Input the curve, a point q of order n (approx.), returns q as a rational
+ * point on the curve, or NULL if q is not rational. */
 static GEN
 torspnt(GEN E, GEN q, long n)
 {
@@ -2926,6 +2965,8 @@ best_in_cycle(GEN e, GEN p, long k)
   return (gsigne(d_ellLHS(e,p)) < 0)? invell(e,p): p;
 }
 
+/* <p,q> = E_tors, possibly NULL (= oo), p,q independant unless NULL
+ * order p = k, order q = 2 unless NULL */
 static GEN
 tors(GEN e, long k, GEN p, GEN q, GEN v)
 {
@@ -2941,7 +2982,6 @@ tors(GEN e, long k, GEN p, GEN q, GEN v)
     p = best_in_cycle(e,p,k);
     if (v)
     {
-      v[1] = linv((GEN)v[1]);
       p = pointch(p,v);
       q = pointch(q,v);
     }
@@ -2956,10 +2996,7 @@ tors(GEN e, long k, GEN p, GEN q, GEN v)
     {
       p = best_in_cycle(e,p,k);
       if (v)
-      {
-        v[1] = linv((GEN)v[1]);
         p = pointch(p,v);
-      }
       r = cgetg(4,t_VEC);
       r[1] = lstoi(k); p1 = cgetg(2,t_VEC); p1[1] = r[1];
       r[2] = (long)p1; p1 = cgetg(2,t_VEC); p1[1] = lcopy(p);
@@ -2979,45 +3016,42 @@ tors(GEN e, long k, GEN p, GEN q, GEN v)
 GEN
 torselldoud(GEN e)
 {
-  long b,i,ord,av=avma,prec, k = 1;
+  long b, i, ord, prec, k = 1;
+  ulong av=avma;
   GEN v,w1,w22,w1j,w12,p,tor1,tor2;
 
-  checkbell(e);
   v = ellintegralmodel(e);
   if (v) e = coordch(e,v);
 
-  b = lgefint((GEN)e[12]) >> 1; /* b = size of sqrt(D) */
-  prec = precision((GEN)e[15]);
-  if (prec < b) err(talker, "precision too low in torselldoud");
-  b = max(b, DEFAULTPREC);
-  if (b < prec) { prec = b; e = gprec_w(e, b); }
-  b = torsbound(e,3);
-  if (b==1) { avma=av; return tors(e,1,NULL,NULL, v); }
-  w22 = gmul2n((GEN)e[16],-1);
+  b = DEFAULTPREC + ((lgefint((GEN)e[12])-2) >> 1); /* b >= size of sqrt(D) */
   w1 = (GEN)e[15];
+  prec = precision(w1);
+  if (prec < b) err(talker, "precision too low in torselldoud");
+  if (b < prec) { prec = b; e = gprec_w(e, b); w1 = (GEN)e[15]; }
+  b = torsbound(e);
+  if (b==1) { avma=av; return tors(e,1,NULL,NULL, v); }
+  if (v) v[1] = linv((GEN)v[1]);
+  w22 = gmul2n((GEN)e[16],-1);
   if (b % 4)
   {
+    p = NULL;
     for (i=10; i>1; i--)
     {
-      if (b%i==0)
+      if (b%i != 0) continue;
+      w1j = gdivgs(w1,i);
+      p = torspnt(e,pointell(e,w1j,prec),i);
+      if (!p && i%2==0)
       {
-	w1j = gdivgs(w1,i);
-	p = torspnt(e,pointell(e,w1j,prec),i);
-	if (i%2==0 && p==NULL)
-	{
-	  p = torspnt(e,pointell(e,gadd(w22,w1j),prec),i);
-	  if (p==NULL)
-	    p = torspnt(e,pointell(e,gadd(w22,gmul2n(w1j,1)),prec),i);
-	}
+        p = torspnt(e,pointell(e,gadd(w22,w1j),prec),i);
+        if (!p) p = torspnt(e,pointell(e,gadd(w22,gmul2n(w1j,1)),prec),i);
       }
-      else p = NULL;
-      if (p) {k = i; break; }
+      if (p) { k = i; break; }
     }
     return gerepileupto(av, tors(e,k,p,NULL, v));
   }
 
   ord = 0; tor1 = tor2 = NULL;
-  w12 = gmul2n((GEN)e[15],-1);
+  w12 = gmul2n(w1,-1);
   if ((p = torspnt(e,pointell(e,w12,prec),2)))
   {
     tor1 = p; ord++;
@@ -3028,48 +3062,49 @@ torselldoud(GEN e)
     tor2 = p; ord += 2;
   }
 
+  p = NULL;
   switch(ord)
   {
-    case 0:
+    case 0: /* no point of order 2 */
       for (i=9; i>1; i-=2)
       {
-        w1j=gdivgs((GEN)e[15],i);
-        p = (b%i==0)? torspnt(e,pointell(e,w1j,prec),i): NULL;
+        if (b%i!=0) continue;
+        w1j = gdivgs(w1,i);
+        p = torspnt(e,pointell(e,w1j,prec),i);
         if (p) { k = i; break; }
       }
       break;
 
-    case 1:
+    case 1: /* 1 point of order 2: w1 / 2 */
       for (i=12; i>2; i-=2)
       {
-        w1j=gdivgs((GEN)e[15],i);
-        if (b%i==0)
-        {
-          p = torspnt(e,pointell(e,w1j,prec),i);
-          if (i%4==0 && p==NULL)
-            p = torspnt(e,pointell(e,gadd(w22,w1j),prec),i);
-        }
-        else p = NULL;
+        if (b%i!=0) continue;
+        w1j = gdivgs(w1,i);
+        p = torspnt(e,pointell(e,w1j,prec),i);
+        if (!p && i%4==0)
+          p = torspnt(e,pointell(e,gadd(w22,w1j),prec),i);
         if (p) { k = i; break; }
       }
       if (!p) { p = tor1; k = 2; }
       break;
 
-    case 2:
+    case 2: /* 1 point of order 2: w2 / 2 or (w1+w2) / 2 */
       for (i=5; i>1; i-=2)
       {
-        w1j = gdivgs((GEN)e[15],i);
-        p = (b%i==0)? torspnt(e,pointell(e,gadd(w22,w1j),prec),i+i): NULL;
-        if (p) { k = i+i; break; }
+        if (b%i!=0) continue;
+        w1j = gdivgs(w1,i);
+        p = torspnt(e,pointell(e,gadd(w22,w1j),prec),i+i);
+        if (p) { k = 2*i; break; }
       }
       if (!p) { p = tor2; k = 2; }
       tor2 = NULL; break;
 
-    case 3:
+    case 3: /* 2 points of order 2: w1/2 and w2/2 */
       for (i=8; i>2; i-=2)
       {
-        w1j=gdivgs((GEN)e[15],i);
-        p = (b%(2*i)==0)? torspnt(e,pointell(e,w1j,prec),i): NULL;
+        if (b%(2*i)!=0) continue;
+        w1j = gdivgs(w1,i);
+        p = torspnt(e,pointell(e,w1j,prec),i);
         if (p) { k = i; break; }
       }
       if (!p) { p = tor1; k = 2; }
